use std::sync::mpsc;

use egui::{
    CentralPanel, Context, FontFamily, Label, RichText, ScrollArea, ViewportBuilder, ViewportId,
};

use crate::emulator::{EmulatorClient, EmulatorCommand, SimId};

use super::AppWindow;

pub struct TerminalWindow {
    sim_id: SimId,
    receiver: mpsc::Receiver<String>,
    text: String,
}

const TEXT: &str = "
Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.

Curabitur pretium tincidunt lacus. Nulla gravida orci a odio. Nullam various, turpis et commodo pharetra, est eros bibendum elit, nec luctus magna felis sollicitudin mauris. Integer in mauris eu nibh euismod gravida. Duis ac tellus et risus vulputate vehicula. Donec lobortis risus a elit. Etiam tempor. Ut ullamcorper, ligula eu tempor congue, eros est euismod turpis, id tincidunt sapien risus a quam. Maecenas fermentum consequat mi. Donec fermentum. Pellentesque malesuada nulla a mi. Duis sapien sem, aliquet nec, commodo eget, consequat quis, neque. Aliquam faucibus, elit ut dictum aliquet, felis nisl adipiscing sapien, sed malesuada diam lacus eget erat. Cras mollis scelerisque nunc. Nullam arcu. Aliquam consequat. Curabitur augue lorem, dapibus quis, laoreet et, pretium ac, nisi. Aenean magna nisl, mollis quis, molestie eu, feugiat in, orci. In hac habitasse platea dictumst.";

impl TerminalWindow {
    pub fn new(sim_id: SimId, client: &EmulatorClient) -> Self {
        let (sender, receiver) = mpsc::channel();
        client.send_command(EmulatorCommand::ConnectTerminal(sim_id, sender));
        Self {
            sim_id,
            receiver,
            text: TEXT.to_string(),
        }
    }
}

impl AppWindow for TerminalWindow {
    fn viewport_id(&self) -> ViewportId {
        ViewportId::from_hash_of(format!("terminal-{}", self.sim_id))
    }

    fn sim_id(&self) -> SimId {
        self.sim_id
    }

    fn initial_viewport(&self) -> ViewportBuilder {
        ViewportBuilder::default()
            .with_title(format!("Terminal ({})", self.sim_id))
            .with_inner_size((640.0, 480.0))
    }

    fn show(&mut self, ctx: &Context) {
        if let Ok(text) = self.receiver.try_recv() {
            self.text.push_str(&text);
        }
        CentralPanel::default().show(ctx, |ui| {
            ScrollArea::vertical().show(ui, |ui| {
                ui.add(Label::new(RichText::new(&self.text).family(FontFamily::Monospace)).wrap());
            });
        });
    }
}
